import { Request, Response } from 'express';
import { prisma } from '../utils/prisma';
import bcrypt from 'bcrypt';
import jwt from 'jsonwebtoken';

class AuthController {
  /**
   * Регистрация нового пользователя (доступ только по adminCode)
   */
  static async register(req: Request, res: Response) {
    try {
      const { username, password, adminCode } = req.body;

      // Проверка наличия данных
      if (!username || !password || !adminCode) {
        return res.status(400).json({ message: 'Необходимо указать имя пользователя, пароль и код доступа' });
      }

      // Проверяем adminCode
      if (adminCode !== process.env.ADMIN_CODE) {
        return res.status(403).json({ message: 'Неверный код доступа' });
      }

      // Проверяем, существует ли пользователь
      const existingUser = await prisma.user.findUnique({ where: { username } });
      if (existingUser) {
        return res.status(400).json({ message: 'Пользователь с таким именем уже существует' });
      }

      // Ищем роль "user" (по умолчанию)
      const userRole = await prisma.role.findFirst({ where: { name: 'user' } });
      if (!userRole) return res.status(500).json({ message: 'Роль пользователя не найдена' });

      // Хешируем пароль
      const hashedPassword = await bcrypt.hash(password, 10);

      // Создаем пользователя
      const user = await prisma.user.create({
        data: {
          username,
          password_hash: hashedPassword,
          role_id: userRole.id,
          is_active: true,
        },
        include: {
          role: true,
        },
      });

      // Генерируем JWT токен
      const token = jwt.sign(
        { id: user.id, username: user.username, role: user.role.name },
        process.env.JWT_SECRET as string,
        { expiresIn: '24h' }
      );

      res.status(201).json({
        message: 'Пользователь успешно зарегистрирован',
        token,
        user: {
          id: user.id,
          username: user.username,
          role: user.role.name,
        },
      });
    } catch (error) {
      console.error('Ошибка при регистрации:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  /**
   * Авторизация пользователя (логин)
   */
  static async login(req: Request, res: Response) {
    try {
      const { username, password } = req.body;

      if (!username || !password) {
        return res.status(400).json({ message: 'Необходимо указать имя пользователя и пароль' });
      }

      // Поиск пользователя
      const user = await prisma.user.findUnique({
        where: { username },
        include: { role: true },
      });

      if (!user) {
        return res.status(401).json({ message: 'Неверное имя пользователя или пароль' });
      }

      // Проверка активности
      if (!user.is_active) {
        return res.status(403).json({ message: 'Учетная запись деактивирована' });
      }

      // Проверка пароля
      const isValidPassword = await bcrypt.compare(password, user.password_hash);
      if (!isValidPassword) {
        return res.status(401).json({ message: 'Неверное имя пользователя или пароль' });
      }

      // Обновляем last_login
      await prisma.user.update({
        where: { id: user.id },
        data: { last_login: new Date() },
      });

      // Генерация токена
      const token = jwt.sign(
        { id: user.id, username: user.username, role: user.role.name },
        process.env.JWT_SECRET as string,
        { expiresIn: '24h' }
      );

      res.json({
        token,
        user: {
          id: user.id,
          username: user.username,
          role: user.role.name,
        },
      });
    } catch (error) {
      console.error('Ошибка при входе:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  /**
   * Валидация JWT токена
   */
  static async validateToken(req: Request, res: Response) {
    try {
      const token = req.headers.authorization?.split(' ')[1];
      if (!token) {
        return res.status(401).json({ message: 'Токен не предоставлен' });
      }
      const secretKey: string = process.env.JWT_SECRET as string;
      const decoded = jwt.verify(token, secretKey);
      res.json({ valid: true, user: decoded });
    } catch (error) {
      res.status(401).json({ valid: false, message: 'Недействительный токен' });
    }
  }
}

export default AuthController;
