import { Request, Response } from 'express';
import { prisma } from '../utils/prisma';
import { Prisma } from '@prisma/client';

// Типы для расширенных моделей с включенными связями
type ItemWithRelations = Prisma.ItemGetPayload<{
  include: {
    category: true;
    box_items: {
      include: {
        box: true;
      };
    };
  };
}>;

type BoxWithRelations = Prisma.BoxGetPayload<{
  include: {
    category: true;
    items: {
      include: {
        item: true;
      };
    };
  };
}>;

// Типы для входящих данных
interface CreateItemData {
  name: string;
  code?: string;
  description?: string;
  categoryId: number;
  quantity?: number;
}

interface UpdateItemData {
  name?: string;
  code?: string;
  description?: string;
  quantity?: number;
}

interface BoxItemInput {
  id: number;
  quantity: number;
}

interface CreateBoxData {
  name: string;
  categoryId: number;
  items: BoxItemInput[];
}

interface UpdateBoxData {
  name?: string;
  items: BoxItemInput[];
}

class InventoryController {
  // Получение товаров категории
  static async getCategoryItems(req: Request, res: Response) {
    try {
      const categoryId = parseInt(req.params.categoryId, 10);
      if (isNaN(categoryId)) {
        return res.status(400).json({ message: 'Некорректный идентификатор категории' });
      }

      const items: ItemWithRelations[] = await prisma.item.findMany({
        where: { category_id: categoryId },
        include: {
          category: true,
          box_items: { include: { box: true } },
        },
      });

      res.json(
        items.map((item) => ({
          id: item.id,
          name: item.name,
          code: item.code,
          description: item.description,
          quantity: item.quantity,
          category: item.category?.name || 'Без категории',
          boxes: item.box_items.map((bi) => ({
            id: bi.box.id,
            name: bi.box.name,
            quantity: bi.quantity,
          })),
        }))
      );
    } catch (error) {
      console.error('Ошибка при получении товаров категории:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  // Создание нового товара
  static async createItem(req: Request<{}, {}, CreateItemData>, res: Response) {
    try {
      const { name, code, description, categoryId, quantity = 0 } = req.body;

      if (!name || !categoryId) {
        return res.status(400).json({ message: 'Необходимо указать название и категорию' });
      }

      const category_id = parseInt(String(categoryId), 10);
      if (isNaN(category_id)) {
        return res.status(400).json({ message: 'Некорректный идентификатор категории' });
      }

      const item = await prisma.item.create({
        data: { name, code, description, quantity, category_id },
        include: { category: true },
      });

      res.json(item);
    } catch (error) {
      console.error('Ошибка при создании товара:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  // Обновление товара
  static async updateItem(req: Request<{ id: string }, {}, UpdateItemData>, res: Response) {
    try {
      const id = parseInt(req.params.id, 10);
      if (isNaN(id)) {
        return res.status(400).json({ message: 'Некорректный идентификатор товара' });
      }

      const item: ItemWithRelations = await prisma.item.update({
        where: { id },
        data: req.body,
        include: {
          category: true,
          box_items: { include: { box: true } },
        },
      });

      res.json(item);
    } catch (error) {
      if (error instanceof Prisma.PrismaClientKnownRequestError) {
        if (error.code === 'P2025') {
          return res.status(404).json({ message: 'Товар не найден' });
        }
      }
      console.error('Ошибка при обновлении товара:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  // Удаление товара
  static async deleteItem(req: Request, res: Response) {
    try {
      const id = parseInt(req.params.id, 10);
      if (isNaN(id)) {
        return res.status(400).json({ message: 'Некорректный идентификатор товара' });
      }

      const boxItems = await prisma.boxItem.findMany({ where: { item_id: id } });

      if (boxItems.length > 0) {
        return res.status(400).json({ message: 'Невозможно удалить товар, который находится в коробках' });
      }

      await prisma.item.delete({ where: { id } });

      res.json({ message: 'Товар успешно удален' });
    } catch (error) {
      console.error('Ошибка при удалении товара:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  // Получение коробок категории
  static async getCategoryBoxes(req: Request, res: Response) {
    try {
      const categoryId = parseInt(req.params.categoryId, 10);
      if (isNaN(categoryId)) {
        return res.status(400).json({ message: 'Некорректный идентификатор категории' });
      }

      const boxes: BoxWithRelations[] = await prisma.box.findMany({
        where: { category_id: categoryId },
        include: {
          category: true,
          items: { include: { item: true } },
        },
      });

      res.json(
        boxes.map(box => ({
          id: box.id,
          name: box.name,
          category: box.category?.name,
          items: box.items.map(bi => ({
            id: bi.item.id,
            name: bi.item.name,
            quantity: bi.quantity,
          })),
        }))
      );
    } catch (error) {
      console.error('Ошибка при получении коробок категории:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  // Создание новой коробки
  static async createBox(req: Request<{}, {}, CreateBoxData>, res: Response) {
    try {
      const { name, categoryId, items } = req.body;

      if (!name || !categoryId || !Array.isArray(items)) {
        return res.status(400).json({ message: 'Некорректные входные данные' });
      }

      const box: BoxWithRelations = await prisma.box.create({
        data: {
          name,
          category_id: categoryId,
          items: {
            create: items.map((item: BoxItemInput) => ({
              item_id: item.id,
              quantity: item.quantity,
            })),
          },
        },
        include: {
          category: true,
          items: { include: { item: true } },
        },
      });

      res.json(box);
    } catch (error) {
      console.error('Ошибка при создании коробки:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  // Обновление коробки
  static async updateBox(req: Request<{ id: string }, {}, UpdateBoxData>, res: Response) {
    try {
      const id = parseInt(req.params.id, 10);
      if (isNaN(id)) {
        return res.status(400).json({ message: 'Некорректный идентификатор коробки' });
      }

      const { name, items } = req.body;

      const box = await prisma.$transaction(async (prisma) => {
        await prisma.boxItem.deleteMany({ where: { box_id: id } });

        return prisma.box.update({
          where: { id },
          data: {
            name,
            items: {
              create: items.map((item: BoxItemInput) => ({
                item_id: item.id,
                quantity: item.quantity,
              })),
            },
          },
          include: {
            category: true,
            items: { include: { item: true } },
          },
        });
      });

      res.json(box);
    } catch (error) {
      console.error('Ошибка при обновлении коробки:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  // Удаление коробки
  static async deleteBox(req: Request, res: Response) {
    try {
      const id = parseInt(req.params.id, 10);
      if (isNaN(id)) {
        return res.status(400).json({ message: 'Некорректный идентификатор коробки' });
      }

      await prisma.box.delete({ where: { id } });

      res.json({ message: 'Коробка успешно удалена' });
    } catch (error) {
      console.error('Ошибка при удалении коробки:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }
}

export default InventoryController;
