import { Request, Response } from 'express';
import { prisma } from '../utils/prisma';

class LocationController {
  static async getAll(req: Request, res: Response) {
    try {
      const locations = await prisma.location.findMany({
        orderBy: { name: 'asc' }
      });
      res.json(locations);
    } catch (error) {
      console.error('Ошибка при получении локаций:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  static async create(req: Request, res: Response) {
    try {
      const { name, description } = req.body;

      if (!name) {
        return res.status(400).json({ message: 'Название локации обязательно' });
      }

      // Проверяем уникальность имени
      const existingLocation = await prisma.location.findFirst({
        where: { name }
      });

      if (existingLocation) {
        return res.status(400).json({ message: 'Локация с таким названием уже существует' });
      }

      const location = await prisma.location.create({
        data: {
          name,
          description
        }
      });

      res.status(201).json(location);
    } catch (error) {
      console.error('Ошибка при создании локации:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  static async update(req: Request, res: Response) {
    try {
      const { id } = req.params;
      const { name, description } = req.body;

      if (!name) {
        return res.status(400).json({ message: 'Название локации обязательно' });
      }

      // Проверяем существование локации
      const existingLocation = await prisma.location.findUnique({
        where: { id: Number(id) }
      });

      if (!existingLocation) {
        return res.status(404).json({ message: 'Локация не найдена' });
      }

      // Проверяем уникальность нового имени
      const duplicateName = await prisma.location.findFirst({
        where: {
          name,
          NOT: { id: Number(id) }
        }
      });

      if (duplicateName) {
        return res.status(400).json({ message: 'Локация с таким названием уже существует' });
      }

      const location = await prisma.location.update({
        where: { id: Number(id) },
        data: { name, description }
      });

      res.json(location);
    } catch (error) {
      console.error('Ошибка при обновлении локации:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  static async delete(req: Request, res: Response) {
    try {
      const { id } = req.params;

      // Проверяем существование локации
      const location = await prisma.location.findUnique({
        where: { id: Number(id) },
        include: { categories: true }
      });

      if (!location) {
        return res.status(404).json({ message: 'Локация не найдена' });
      }

      // Проверяем, есть ли связанные категории
      if (location.categories.length > 0) {
        return res.status(400).json({ 
          message: 'Невозможно удалить локацию, содержащую категории' 
        });
      }

      await prisma.location.delete({
        where: { id: Number(id) }
      });

      res.json({ message: 'Локация успешно удалена' });
    } catch (error) {
      console.error('Ошибка при удалении локации:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }
}

export default LocationController; 