import { Request, Response } from 'express';
import ReportService from '../services/ReportService';
import { prisma } from '../utils/prisma';

class ReportController {
  static async generateInventoryReport(req: Request, res: Response) {
    try {
      const { categoryId, format = 'excel' } = req.query;
      const parsedCategoryId = categoryId ? parseInt(categoryId as string, 10) : undefined;

      if (categoryId && isNaN(parsedCategoryId!)) {
        return res.status(400).json({ message: 'Некорректный идентификатор категории' });
      }

      let buffer: Buffer;
      if (format === 'excel') {
        buffer = await ReportService.generateInventoryReport(parsedCategoryId);
        res.setHeader('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        res.setHeader('Content-Disposition', 'attachment; filename=inventory-report.xlsx');
      } else if (format === 'pdf') {
        const items = await prisma.item.findMany({
          // Используем parsedCategoryId вместо categoryId
          where: parsedCategoryId !== undefined ? { category_id: parsedCategoryId } : undefined,
          include: {
            category: {
              include: {
                location: true
              }
            },
            box_items: {
              include: {
                box: true
              }
            }
          }
        });
        buffer = await ReportService.generatePDFReport(items, 'inventory');
        res.setHeader('Content-Type', 'application/pdf');
        res.setHeader('Content-Disposition', 'attachment; filename=inventory-report.pdf');
      } else {
        return res.status(400).json({ message: 'Неподдерживаемый формат отчета' });
      }

      res.send(buffer);
    } catch (error) {
      console.error('Ошибка при генерации отчета по инвентарю:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  static async generateBoxReport(req: Request, res: Response) {
    try {
      const { categoryId, format = 'excel' } = req.query;

      if (!categoryId) {
        return res.status(400).json({ message: 'Необходимо указать категорию' });
      }

      const parsedCategoryId = parseInt(categoryId as string, 10);
      if (isNaN(parsedCategoryId)) {
        return res.status(400).json({ message: 'Некорректный идентификатор категории' });
      }

      let buffer: Buffer;
      if (format === 'excel') {
        buffer = await ReportService.generateBoxReport(parsedCategoryId);
        res.setHeader('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        res.setHeader('Content-Disposition', 'attachment; filename=box-report.xlsx');
      } else if (format === 'pdf') {
        const boxes = await prisma.box.findMany({
          where: { category_id: parsedCategoryId },
          include: {
            category: {
              include: {
                location: true
              }
            },
            items: {
              include: {
                item: true
              }
            }
          }
        });
        buffer = await ReportService.generatePDFReport(boxes, 'boxes');
        res.setHeader('Content-Type', 'application/pdf');
        res.setHeader('Content-Disposition', 'attachment; filename=box-report.pdf');
      } else {
        return res.status(400).json({ message: 'Неподдерживаемый формат отчета' });
      }

      res.send(buffer);
    } catch (error) {
      console.error('Ошибка при генерации отчета по коробкам:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }

  static async generateYearEndReport(req: Request, res: Response) {
    try {
      const { year, format = 'excel' } = req.query;

      if (!year) {
        return res.status(400).json({ message: 'Необходимо указать год' });
      }

      const parsedYear = parseInt(year as string, 10);
      if (isNaN(parsedYear)) {
        return res.status(400).json({ message: 'Некорректный формат года' });
      }

      const buffer = await ReportService.generateYearEndReport(parsedYear);

      res.setHeader('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
      res.setHeader('Content-Disposition', `attachment; filename=year-end-report-${parsedYear}.xlsx`);
      res.send(buffer);
    } catch (error) {
      console.error('Ошибка при генерации годового отчета:', error);
      res.status(500).json({ message: 'Ошибка сервера' });
    }
  }
}

export default ReportController;
