import { prisma } from '../utils/prisma';
import { Prisma } from '@prisma/client';
import ExcelJS from 'exceljs';
import PDFDocument from 'pdfkit';
import { Buffer } from 'buffer';

type ItemWithRelations = Prisma.ItemGetPayload<{
  include: {
    category: {
      include: {
        location: true;
      };
    };
    box_items: {
      include: {
        box: true;
      };
    };
  };
}>;

type BoxWithRelations = Prisma.BoxGetPayload<{
  include: {
    category: {
      include: {
        location: true;
      };
    };
    items: {
      include: {
        item: true;
      };
    };
  };
}>;

class ReportService {
  static async generateInventoryReport(categoryId?: number): Promise<Buffer> {
    try {
      const workbook = new ExcelJS.Workbook();
      const worksheet = workbook.addWorksheet('Инвентарь');

      // Настройка заголовков
      worksheet.columns = [
        { header: 'ID', key: 'id', width: 10 },
        { header: 'Название', key: 'name', width: 30 },
        { header: 'Код', key: 'code', width: 15 },
        { header: 'Описание', key: 'description', width: 40 },
        { header: 'Количество', key: 'quantity', width: 15 },
        { header: 'Категория', key: 'category', width: 20 },
        { header: 'Коробки', key: 'boxes', width: 40 },
      ];

      // Получение данных
      const items: ItemWithRelations[] = await prisma.item.findMany({
        where: categoryId ? { category_id: categoryId } : undefined,
        include: {
          category: {
            include: {
              location: true,
            },
          },
          box_items: {
            include: {
              box: true,
            },
          },
        },
      });

      // Заполнение данными
      items.forEach((item) => {
        worksheet.addRow({
          id: item.id,
          name: item.name,
          code: item.code || '',
          description: item.description || '',
          quantity: item.quantity,
          category: item.category?.name || 'Без категории',
          boxes: item.box_items
            .map((bi) => `${bi.box.name} (${bi.quantity})`)
            .join(', '),
        });
      });

      // Стилизация
      const headerRow = worksheet.getRow(1);
      headerRow.font = { bold: true };
      headerRow.alignment = { vertical: 'middle', horizontal: 'center' };

      // Возвращаем буфер
      return await workbook.xlsx.writeBuffer() as Buffer;
    } catch (error) {
      console.error('Ошибка при генерации отчета по инвентарю:', error);
      throw error;
    }
  }

  static async generateBoxReport(categoryId: number): Promise<Buffer> { // Исправленный тип
    try {
      const workbook = new ExcelJS.Workbook();
      const worksheet = workbook.addWorksheet('Коробки');

      // Настройка заголовков
      worksheet.columns = [
        { header: 'ID коробки', key: 'boxId', width: 10 },
        { header: 'Название коробки', key: 'boxName', width: 30 },
        { header: 'Категория', key: 'category', width: 20 },
        { header: 'Товар', key: 'itemName', width: 30 },
        { header: 'Код товара', key: 'itemCode', width: 15 },
        { header: 'Количество', key: 'quantity', width: 15 },
      ];

      // Получение данных
      const boxes: BoxWithRelations[] = await prisma.box.findMany({
        where: { category_id: categoryId },
        include: {
          category: {
            include: {
              location: true,
            },
          },
          items: {
            include: {
              item: true,
            },
          },
        },
      });

      // Заполнение данными
      boxes.forEach((box) => {
        box.items.forEach((boxItem) => {
          worksheet.addRow({
            boxId: box.id,
            boxName: box.name,
            category: box.category?.name || 'Без категории',
            itemName: boxItem.item.name,
            itemCode: boxItem.item.code || '',
            quantity: boxItem.quantity,
          });
        });
      });

      // Стилизация
      const headerRow = worksheet.getRow(1);
      headerRow.font = { bold: true };
      headerRow.alignment = { vertical: 'middle', horizontal: 'center' };

      // Возвращаем буфер
      return await workbook.xlsx.writeBuffer() as Buffer;
    } catch (error) {
      console.error('Ошибка при генерации отчета по коробкам:', error);
      throw error;
    }
  }

  static async generateYearEndReport(year: number): Promise<Buffer> {
    try {
      const workbook = new ExcelJS.Workbook();
      const worksheet = workbook.addWorksheet(`Остатки ${year}`);

      // Настройка заголовков
      worksheet.columns = [
        { header: 'Категория', key: 'category', width: 30 },
        { header: 'Товар', key: 'name', width: 30 },
        { header: 'Код', key: 'code', width: 15 },
        { header: 'Описание', key: 'description', width: 40 },
        { header: 'Количество', key: 'quantity', width: 15 },
      ];

      // Получение данных
      const items: ItemWithRelations[] = await prisma.item.findMany({
        where: {
          category: {
            name: {
              startsWith: `${year} Остатки`,
            },
          },
        },
        include: {
          category: {
            include: {
              location: true,
            },
          },
          box_items: {
            include: {
              box: true,
            },
          },
        },
      });

      // Заполнение данными
      items.forEach((item) => {
        worksheet.addRow({
          category: item.category?.name || 'Без категории',
          name: item.name,
          code: item.code || '',
          description: item.description || '',
          quantity: item.quantity,
        });
      });

      // Стилизация
      const headerRow = worksheet.getRow(1);
      headerRow.font = { bold: true };
      headerRow.alignment = { vertical: 'middle', horizontal: 'center' };

      // Возвращаем буфер
      return await workbook.xlsx.writeBuffer() as Buffer;
    } catch (error) {
      console.error('Ошибка при генерации годового отчета:', error);
      throw error;
    }
  }

  static async generateExcelReport(
    data: ItemWithRelations[] | BoxWithRelations[],
    type: 'inventory' | 'boxes'
  ): Promise<Buffer> {
    const workbook = new ExcelJS.Workbook();
    const worksheet = workbook.addWorksheet('Report');

    if (type === 'inventory') {
      // Настройка колонок для инвентарного отчета
      worksheet.columns = [
        { header: 'Локация', key: 'location', width: 15 },
        { header: 'Категория', key: 'category', width: 20 },
        { header: 'Код', key: 'code', width: 15 },
        { header: 'Название', key: 'name', width: 30 },
        { header: 'Количество', key: 'quantity', width: 12 },
      ];

      // Заполнение данными
      (data as ItemWithRelations[]).forEach((item) => {
        worksheet.addRow({
          location: item.category?.location?.name || 'Без локации',
          category: item.category?.name || 'Без категории',
          code: item.code || '',
          name: item.name,
          quantity: item.quantity,
        });
      });
    } else {
      // Настройка колонок для отчета по коробкам
      worksheet.columns = [
        { header: 'Локация', key: 'location', width: 15 },
        { header: 'Категория', key: 'category', width: 20 },
        { header: 'Коробка', key: 'box', width: 20 },
        { header: 'Товар', key: 'item', width: 30 },
        { header: 'Количество', key: 'quantity', width: 12 },
      ];

      // Заполнение данными
      (data as BoxWithRelations[]).forEach((box) => {
        box.items.forEach((item) => {
          worksheet.addRow({
            location: box.category?.location?.name || 'Без локации',
            category: box.category?.name || 'Без категории',
            box: box.name,
            item: item.item.name,
            quantity: item.quantity,
          });
        });
      });
    }

    // Стилизация заголовков
    const headerRow = worksheet.getRow(1);
    headerRow.font = { bold: true };
    headerRow.fill = {
      type: 'pattern',
      pattern: 'solid',
      fgColor: { argb: 'FFE0E0E0' },
    };
    headerRow.alignment = { vertical: 'middle', horizontal: 'center' };

    return await workbook.xlsx.writeBuffer() as Buffer;
  }

  static async generatePDFReport(
    data: ItemWithRelations[] | BoxWithRelations[],
    type: 'inventory' | 'boxes'
  ): Promise<Buffer> {
    return new Promise((resolve, reject) => {
      const chunks: Buffer[] = [];
      const doc = new PDFDocument({ margin: 50 });

      doc.on('data', (chunk) => chunks.push(chunk));
      doc.on('end', () => resolve(Buffer.concat(chunks)));
      doc.on('error', reject);

      // Добавляем заголовок
      doc
        .fontSize(16)
        .text(
          type === 'inventory' ? 'Инвентарный отчет' : 'Отчет по коробкам',
          { align: 'center' }
        );
      doc.moveDown();

      if (type === 'inventory') {
        // Группируем данные по локациям и категориям
        const grouped = (data as ItemWithRelations[]).reduce(
          (acc, item) => {
            const locationName = item.category?.location?.name || 'Без локации';
            const categoryName = item.category?.name || 'Без категории';

            if (!acc[locationName]) {
              acc[locationName] = {};
            }
            if (!acc[locationName][categoryName]) {
              acc[locationName][categoryName] = [];
            }

            acc[locationName][categoryName].push(item);
            return acc;
          },
          {} as Record<string, Record<string, ItemWithRelations[]>>
        );

        // Выводим данные
        Object.entries(grouped).forEach(([location, categories]) => {
          doc.fontSize(14).text(location, { underline: true });
          doc.moveDown(0.5);

          Object.entries(categories).forEach(([category, items]) => {
            doc.font('Helvetica-Bold').fontSize(12).text(category);
            doc.moveDown(0.5);

            items.forEach((item) => {
              doc.fontSize(10).text(
                `${item.code || 'Без кода'} - ${item.name}: ${item.quantity} шт.`,
                { indent: 20 }
              );
            });
            doc.moveDown();
          });
          doc.moveDown();
        });
      } else {
        // Вывод данных по коробкам
        (data as BoxWithRelations[]).forEach((box) => {
          doc
            .fontSize(12)
            .text(
              `${box.category?.location?.name || 'Без локации'} - ${
                box.category?.name || 'Без категории'
              }`,
              { underline: true }
            );
          doc.fontSize(11).text(`Коробка: ${box.name}`);
          doc.moveDown(0.5);

          box.items.forEach((item) => {
            doc.fontSize(10).text(`${item.item.name}: ${item.quantity} шт.`, { indent: 20 });
          });
          doc.moveDown();
        });
      }

      doc.end();
    });
  }
}

export default ReportService;
