import { PrismaClient } from '@prisma/client';
import { scheduleJob } from 'node-schedule';

const prisma = new PrismaClient();

class YearEndService {
  // Запускаем планировщик для автоматического переноса
  static initScheduler() {
    // Запускаем задачу в последний день года в 23:59
    scheduleJob('59 23 31 12 *', async () => {
      try {
        await YearEndService.transferRemainders();
      } catch (error) {
        console.error('Error in year-end transfer:', error);
      }
    });
  }

  // Метод для ручного запуска переноса остатков
  static async transferRemainders() {
    const currentYear = new Date().getFullYear();
    const nextYear = currentYear + 1;

    try {
      await prisma.$transaction(async (prisma) => {
        // Получаем все локации
        const locations = await prisma.location.findMany();

        for (const location of locations) {
          // Получаем все категории для локации
          const categories = await prisma.category.findMany({
            where: { location_id: location.id }
          });

          for (const category of categories) {
            // Создаем новую категорию для следующего года
            const newCategory = await prisma.category.create({
              data: {
                name: `${nextYear} Остатки - ${category.name}`,
                location_id: location.id
              }
            });

            // Получаем все товары из текущей категории
            const items = await prisma.item.findMany({
              where: { category_id: category.id }
            });

            // Переносим товары с остатками в новую категорию
            for (const item of items) {
              if (item.quantity > 0) {
                await prisma.item.create({
                  data: {
                    name: item.name,
                    code: item.code,
                    description: `Перенесено из ${currentYear} года - ${item.description || ''}`,
                    quantity: item.quantity,
                    category_id: newCategory.id
                  }
                });

                // Обнуляем количество в старой записи
                await prisma.item.update({
                  where: { id: item.id },
                  data: { quantity: 0 }
                });
              }
            }
          }
        }

        // Создаем запись в журнале переносов
        await prisma.yearEndTransfer.create({
          data: {
            year: currentYear,
            completed_at: new Date(),
            status: 'completed'
          }
        });
      });

      console.log(`Year-end transfer for ${currentYear} completed successfully`);
    } catch (error) {
      console.error('Error during year-end transfer:', error);
      throw error;
    }
  }

  // Метод для проверки статуса последнего переноса
  static async getLastTransferStatus() {
    try {
      const lastTransfer = await prisma.yearEndTransfer.findFirst({
        orderBy: { completed_at: 'desc' }
      });
      return lastTransfer;
    } catch (error) {
      console.error('Error getting last transfer status:', error);
      throw error;
    }
  }
}

export default YearEndService; 